/*--------------------------------------------------------------* 
 *   Name: Jeffrey J. Weston
 *   Assignment: Final Project
 *   Email: westonje@engr.orst.edu
 *   Reference examples/material: Assignment #3, Assignment #4
 *--------------------------------------------------------------*/

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <GL/glut.h>
#include <GL/gl.h>
#include <stdlib.h>
#include <math.h>
#include <iostream.h>

#define PI          3.14159
#define ANGLE_STEPS 1000

// Camera Position
#define X           0
#define Y           0
#define Z           55

int iStep, iCurX, iCurY, iCurPlayer, iMode, iExpStep, iAngMode;
// iStep : Legacy Code. It determines at what angle the camera is at
// iCurX : The X value of the currently selected square
// iCurY : The Y value of the currently selected square
// iCurPlayer : The player whose turn it is
// iMode : Determines what state the game is in. States listed below
// iExpStep : Determines at what point we are in an explosion

// ** iMode Settings **
// 0 = Waiting for player input
// 1 = Running explosions -- expansion phase
// 2 = Running explosions -- collapsion phase
// 3 = Red has won
// 4 = Blue has won

int iMaxBombs[8][8] = {{2, 3, 3, 3, 3, 3, 3, 2},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {3, 4, 4, 4, 4, 4, 4, 3},
                       {2, 3, 3, 3, 3, 3, 3, 2}};
// Lookup table of number of bombs each square can hold

int iBombCount[8][8];
// How many bombs are in each square

int iExploding[8][8];
// =1 if a sqaure is exploding. =0 otherwise

int iPlayer[8][8];
// Which player "owns" that square
// 0 = no one
// 1 = red
// 2 = blue

float fX, fY, fZ;
// Used to compute camera position


void DisplayMainWin ();


// Calculate the X component of a normal. Takes 3 XYZ points as arguments
//  1. Determine 2 vectors from the points
//  2. Calculate the X factor cross product
double CalcNormalX (double x1, double y1, double z1,
                    double x2, double y2, double z2,
                    double x3, double y3, double z3)
{
  double vx1, vy1, vz1, vx2, vy2, vz2, result;

  vx1 = x2 - x1;
  vy1 = y2 - y1;
  vz1 = z2 - z1;

  vx2 = x2 - x3;
  vy2 = y2 - y3;
  vz2 = z2 - z3;

  result = vy1 * vz2 - vz1 * vy2; 

  return (result);
}

// Calculate the Y component of a normal. Takes 3 XYZ points as arguments
//  1. Determine 2 vectors from the points
//  2. Calculate the Y factor cross product
double CalcNormalY (double x1, double y1, double z1,
                    double x2, double y2, double z2,
                    double x3, double y3, double z3)
{
  double vx1, vy1, vz1, vx2, vy2, vz2, result;

  vx1 = x2 - x1;
  vy1 = y2 - y1;
  vz1 = z2 - z1;

  vx2 = x2 - x3;
  vy2 = y2 - y3;
  vz2 = z2 - z3;

  result = vz1 * vx2 - vx1 * vz2;

  return (result);
}

// Calculate the Z component of a normal. Takes 3 XYZ points as arguments
//  1. Determine 2 vectors from the points
//  2. Calculate the Z factor cross product
double CalcNormalZ (double x1, double y1, double z1,
                    double x2, double y2, double z2,
                    double x3, double y3, double z3)
{
  double vx1, vy1, vz1, vx2, vy2, vz2, result;

  vx1 = x2 - x1;
  vy1 = y2 - y1;
  vz1 = z2 - z1;

  vx2 = x2 - x3;
  vy2 = y2 - y3;
  vz2 = z2 - z3;


  result = vx1 * vy2 - vy1 * vx2;

  return (result);
}

// Draw a OpenGL triangle with correct normal. Takes 3 XYZ points as arguments
//  1. Determine normal vector
//  2. Draw the triangle
void MyNormalTriangle (float fX1, float fY1, float fZ1,
                       float fX2, float fY2, float fZ2,
                       float fX3, float fY3, float fZ3)
{
  float nX, nY, nZ;

  nX = CalcNormalX (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);
  nY = CalcNormalY (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);
  nZ = CalcNormalZ (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);

  glBegin (GL_TRIANGLES);

  glNormal3f (nX, nY, nZ);

  glVertex3f (fX1, fY1, fZ1);
  glVertex3f (fX2, fY2, fZ2);
  glVertex3f (fX3, fY3, fZ3);

  glEnd ();
}

// Draw a OpenGL quad with correct normal. Takes 4 XYZ points as arguments
//  1. Determine normal vector
//  2. Draw the quad
void MyNormalQuad (float fX1, float fY1, float fZ1,
                   float fX2, float fY2, float fZ2,
                   float fX3, float fY3, float fZ3,
                   float fX4, float fY4, float fZ4)
{
  float nX, nY, nZ;

  nX = CalcNormalX (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);
  nY = CalcNormalY (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);
  nZ = CalcNormalZ (fX1, fY1, fZ1, fX2, fY2, fZ2, fX3, fY3, fZ3);

  glBegin (GL_QUADS);

  glNormal3f (-nX, -nY, -nZ);

  glVertex3f (fX1, fY1, fZ1);
  glVertex3f (fX2, fY2, fZ2);
  glVertex3f (fX3, fY3, fZ3);
  glVertex3f (fX4, fY4, fZ4);

  glEnd ();
}

// Draw all 6 sides for a cube with all sides being variable
// Takes one XYZ point for a starting point and 3 side lengths as arguments
// - Revised for final project to follow right hand rule
void MyRect (float fX, float fY, float fZ, float fXs, float fYs, float fZs)
{

  MyNormalQuad (fX, fY, fZ,
                fX + fXs, fY, fZ,
                fX + fXs, fY + fYs, fZ,
                fX, fY + fYs, fZ);

  MyNormalQuad (fX, fY, fZ + fZs,
                fX + fXs, fY, fZ + fZs,
                fX + fXs, fY, fZ,
                fX, fY, fZ);
                
  MyNormalQuad (fX, fY, fZ,
                fX, fY + fYs, fZ,
                fX, fY + fYs, fZ + fZs,
                fX, fY, fZ + fZs);

  MyNormalQuad (fX + fXs, fY, fZ + fZs,
                fX, fY, fZ + fZs,
                fX, fY + fYs, fZ + fZs,
                fX + fXs, fY + fYs, fZ + fZs);

  MyNormalQuad (fX + fXs, fY + fYs, fZ + fZs,
                fX, fY + fYs, fZ + fZs,
                fX, fY + fYs, fZ,
                fX + fXs, fY + fYs, fZ);

  MyNormalQuad (fX + fXs, fY + fYs, fZ,
                fX + fXs, fY, fZ,
                fX + fXs, fY, fZ + fZs,
                fX + fXs, fY + fYs, fZ + fZs);
}

// Draw a sphere with a variable radius and detail level
// Takes one XYZ point for a center point, a radius,
// and detail level as arguments
void MyRealSphere (float fX, float fY, float fZ, float fR, int iDetail)
{
  int iLoop1, iLoop2;
  float fYlevel1, fYlevel2, fRleft1, fRleft2, fYchunk, fTheta,
        fX1, fX2, fX3, fX4,
        fY1, fY2, fY3, fY4,
        fZ1, fZ2, fZ3, fZ4;

  // Determine the size of each verticle section of the sphere
  fYchunk = (fR * 2) / (iDetail - 1);

  if (iDetail > 2) // Detail level must be 3 or greater to work
  {
    for (iLoop1 = 0; iLoop1 < iDetail; iLoop1++)
    {
      for (iLoop2 = 0; iLoop2 < iDetail * 2; iLoop2++)
      {
        if (iLoop1 == 0) // Special case - Top Segment
        {
          fYlevel1 = fY + fR; // Top of sphere
          fRleft1 = 0;

          fYlevel2 = (fY + fR) - (fYchunk * (iLoop1 + 1));
          // Get the Y value of the next level down

          fRleft2 = sqrt ((fR * fR) - ((fY - fYlevel2) * (fY - fYlevel2)));
          // Get the radius of the circle at that level

          // Point 1
          fX1 = fX;
          fY1 = fYlevel1;
          fZ1 = fZ;

          fTheta = ((2 * PI) / (iDetail * 2)) * iLoop2;
          // Get the angle of the section we are drawing

          // Point 2
          fX2 = fX + (sin (fTheta) * fRleft2);
          fY2 = fYlevel2;
          fZ2 = fZ + (cos (fTheta) * fRleft2);

          // Get the angle of the next section over
          fTheta = ((2 * PI) / (iDetail * 2)) * (iLoop2 + 1);

          // Point 3
          fX3 = fX + (sin (fTheta) * fRleft2);
          fY3 = fYlevel2;
          fZ3 = fZ + (cos (fTheta) * fRleft2);

          // Draw a triange with normal vector
          MyNormalTriangle (fX3, fY3, fZ3,
                            fX2, fY2, fZ2,
                            fX1, fY1, fZ1);
        }
        else if (iLoop1 == iDetail - 1) // Special case - Bottom Segment
        {
          fYlevel1 = fY - fR; // Bottom of sphere
          fRleft1 = 0;

          fYlevel2 = (fY + fR) - (fYchunk * (iLoop1 - 1));
          // Get the Y value of the next level up

          fRleft2 = sqrt ((fR * fR) - ((fY - fYlevel2) * (fY - fYlevel2)));
          // Get the radius of the circle at that level

          // Point 1
          fX1 = fX;
          fY1 = fYlevel1;
          fZ1 = fZ;

          // Get the angle of the section we are drawing
          fTheta = ((2 * PI) / (iDetail * 2)) * iLoop2;

          // Point 2
          fX2 = fX + (sin (fTheta) * fRleft2);
          fY2 = fYlevel2;
          fZ2 = fZ + (cos (fTheta) * fRleft2);

          // Get the angle of the next section over
          fTheta = ((2 * PI) / (iDetail * 2)) * (iLoop2 + 1);

          // Point 3
          fX3 = fX + (sin (fTheta) * fRleft2);
          fY3 = fYlevel2;
          fZ3 = fZ + (cos (fTheta) * fRleft2);

          // Draw a triange with normal vector
          MyNormalTriangle (fX1, fY1, fZ1,
                            fX2, fY2, fZ2,
                            fX3, fY3, fZ3);
        }
        else if (iLoop1 < iDetail - 2) // Everything in between top and bottom
        {
          // Get top level of current section
          fYlevel1 = (fY + fR) - (fYchunk * (iLoop1));

          // Get radius of the circle at that level
          fRleft1 = sqrt ((fR * fR) - ((fY - fYlevel1) * (fY - fYlevel1)));

          // Get bottom level of current section
          fYlevel2 = (fY + fR) - (fYchunk * (iLoop1 + 1));

          // Get radius of the circle at that level
          fRleft2 = sqrt ((fR * fR) - ((fY - fYlevel2) * (fY - fYlevel2)));

          // Get the first angle of the section
          fTheta = ((2 * PI) / (iDetail * 2)) * iLoop2;

          // Point 1
          fX1 = fX + (sin (fTheta) * fRleft1);
          fY1 = fYlevel1;
          fZ1 = fZ + (cos (fTheta) * fRleft1);

          // Point 2
          fX2 = fX + (sin (fTheta) * fRleft2);
          fY2 = fYlevel2;
          fZ2 = fZ + (cos (fTheta) * fRleft2);

          // Get the second angle of the section
          fTheta = ((2 * PI) / (iDetail * 2)) * (iLoop2 + 1);

          // Point 3
          fX3 = fX + (sin (fTheta) * fRleft2);
          fY3 = fYlevel2;
          fZ3 = fZ + (cos (fTheta) * fRleft2);
 
          // Point 4
          fX4 = fX + (sin (fTheta) * fRleft1);
          fY4 = fYlevel1;
          fZ4 = fZ + (cos (fTheta) * fRleft1);

          // Draw a quad with normal vector
          MyNormalQuad (fX1, fY1, fZ1,
                        fX2, fY2, fZ2,
                        fX3, fY3, fZ3,
                        fX4, fY4, fZ4);
        }
      }
    }
  }
}

void MySphere (float fX, float fY, float fZ, float fR, int iDetail)
{
  static int iSphere1 = 0;
  static int iSphere2 = 0;
  static int iSphere3 = 0;

  glTranslatef (fX, fY, fZ);
  glScalef (fR, fR, fR);

  switch (iDetail)
  {
    case 3 :
      if (iSphere1) glCallList (iSphere1);
      else
      {
        iSphere1 = glGenLists (1);
        glNewList (iSphere1, GL_COMPILE_AND_EXECUTE);
        MyRealSphere (0, 0, 0, 1, iDetail);
        glEndList ();
      }
      break;

    case 15 :
      if (iSphere2) glCallList (iSphere2);
      else
      {
        iSphere2 = glGenLists (1);
        glNewList (iSphere2, GL_COMPILE_AND_EXECUTE);
        MyRealSphere (0, 0, 0, 1, iDetail);
        glEndList ();
      }
      break;

    case 100 :
      if (iSphere3) glCallList (iSphere3);
      else
      {
        iSphere3 = glGenLists (1);
        glNewList (iSphere3, GL_COMPILE_AND_EXECUTE);
        MyRealSphere (0, 0, 0, 1, iDetail);
        glEndList ();
      }
      break;

    default :
      MyRealSphere (0, 0, 0, 1, iDetail);
      break;
  }
  glScalef (1 / fR, 1 / fR, 1 / fR);
  glTranslatef (-fX, -fY, -fZ);
}

// Renders everything about the game onto the screen
void MyDisplay ()
{
  int i1, i2, iX, iY, iZ;

  // Setup ambient coefficient of materials
  GLfloat fNrmAmbient[] = {.2, .2, .2, 1.0};
  GLfloat fHghAmbient[] = {.2, .2, .2, 1.0};
  GLfloat fRedAmbient[] = {.3, .2, .2, 1.0};
  GLfloat fBluAmbient[] = {.2, .2, .3, 1.0};
  GLfloat fBrdAmbient[] = {.3, .2, .2, 1.0};
  GLfloat fBblAmbient[] = {.2, .2, .3, 1.0};

  // Setup diffuse coefficient of materials
  GLfloat fNrmDiffuse[] = {.8, .8, .8, 1.0};
  GLfloat fHghDiffuse[] = {.8, 1.0, .8, 1.0};
  GLfloat fRedDiffuse[] = {1.0, .6, .6, 1.0};
  GLfloat fBluDiffuse[] = {.6, .6, 1.0, 1.0};
  GLfloat fBrdDiffuse[] = {1.0, .6, .6, 1.0};
  GLfloat fBblDiffuse[] = {.6, .6, 1.0, 1.0};

  // Setup specular coefficient of materials
  GLfloat fNrmSpeculr[] = {.0, .0, .0, 1.0};
  GLfloat fHghSpeculr[] = {.0, .0, .0, 1.0};
  GLfloat fRedSpeculr[] = {.1, .0, .0, 1.0};
  GLfloat fBluSpeculr[] = {.0, .0, .1, 1.0};
  GLfloat fBrdSpeculr[] = {5.0, .0, .0, 1.0};
  GLfloat fBblSpeculr[] = {.0, .0, 5.0, 1.0};

  if (iMode < 3) // Display gameboard only if mode is appropriate
  {
    for (i1 = 0; i1 < 8; i1++)
      for (i2 = 0; i2 < 8; i2++) // Cycle through all 64 blocks
      {
        iX = -51 + (13 * i1); // Figure the X value of the corner point
        iY = -51 + (13 * i2); // Figure the Y value of the corner point

        // Figure the Z value of the square
        iZ = (iBombCount[i1][i2] - iMaxBombs[i1][i2]) * 3;

        // Determine what color to draw the current square
        if ((i1 == iCurX) && (i2 == iCurY) && (iMode == 0))
        {
          if (iCurPlayer == 1) // if it's currently selected by red
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fRedAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fRedDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fRedSpeculr);
          }
          else                 // if it's currently selected by blue
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBluAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBluDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBluSpeculr);
          }
        }
        else
        {
          if (iBombCount[i1][i2] >= iMaxBombs[i1][i2] - 1)
                               // if it's about ready to explode
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fHghAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fHghDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fHghSpeculr);
          }
          else                 // if it's nothing special
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fNrmAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fNrmDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fNrmSpeculr);
          }
        }

        MyRect (iX, iY, iZ, 11, 11, -1); // Draw the square

        if (iBombCount[i1][i2] > 0)      // Check for bombs
        {
          if (iPlayer[i1][i2] == 1)      // Check color of bombs
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBrdAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBrdDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBrdSpeculr);
          }
          else
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBblAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBblDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBblSpeculr);
          }

          switch (iBombCount[i1][i2])
          {
            case 1 :                     // Placement of 1 bomb
              MySphere (iX + 5.5, iY + 5.5, iZ, 2, 3);
              break;
            case 2 :                     // Placement of 2 bombs
              MySphere (iX + 3, iY + 3, iZ, 2, 3);
              MySphere (iX + 8, iY + 8, iZ, 2, 3);
              break;
            case 3 :                     // Placement of 3 bombs
              MySphere (iX + 3, iY + 3, iZ, 2, 3);
              MySphere (iX + 8, iY + 3, iZ, 2, 3);
              MySphere (iX + 5.5, iY + 8, iZ, 2, 3);
              break;
            default :                    // Placement of 4 bombs
              MySphere (iX + 3, iY + 3, iZ, 2, 3);
              MySphere (iX + 8, iY + 8, iZ, 2, 3);
              MySphere (iX + 8, iY + 3, iZ, 2, 3);
              MySphere (iX + 3, iY + 8, iZ, 2, 3);
          }
        }
        if (iExploding[i1][i2])          // Check for explosions
        {
        if (iCurPlayer == 1)             // Check color of explosions
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBrdAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBrdDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBrdSpeculr);
          }
          else
          {
            glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBblAmbient);
            glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBblDiffuse);
            glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBblSpeculr);
          }
                                         // Draw the explosion
          MySphere (iX + 5.5, iY + 5.5, iZ, (float)iExpStep, 15);
        }
      }
  }
  else                                   // Win condition
  {
    if (iMode == 3)                      // Red has won
    {
      glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBrdAmbient);
      glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBrdDiffuse);
      glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBrdSpeculr);
    }
    else                                 // Blue has won
    {
      glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT,  fBblAmbient);
      glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE,  fBblDiffuse);
      glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, fBblSpeculr);
    }
    MySphere (0, 0, 0, 37, 100);         // Draw the "win" sphere
  }
}

// Checks to see if any squares have enough bombs to explode.
// If they do, the iExploding array is set and 1 is returned.
int DetectExplosions ()
{
  int i1, i2, iResult;

  iResult = 0;

  for (i1 = 0; i1 < 8; i1++)
    for (i2 = 0; i2 < 8; i2++)
    {
      // Check the bomb count against the lookup table
      if (iBombCount[i1][i2] >= iMaxBombs[i1][i2])
      {
        iExploding[i1][i2] = 1;    // Set the iExploding array
        iResult = 1;               // iResult is returned below
      }
      else iExploding[i1][i2] = 0; // Reset the iExploding array

      // If no bombs in square, mark square as un-owned
      if (iBombCount[i1][i2] == 0) iPlayer[i1][i2] = 0;
    }
  return (iResult);
}

// After an explosion, this function is called to adjust bomb locations
void MoveBombs ()
{
  int i1, i2;

  for (i1 = 0; i1 < 8; i1++)
    for (i2 = 0; i2 < 8; i2++)
      if (iExploding[i1][i2])             // Checks if an explosion occured
      {
        if (i1 > 0)                       // If not against the left wall
        {
          iBombCount[i1][i2]--;           // Remove one bomb
          iBombCount[i1-1][i2]++;         // Add one bomb to the left
          iPlayer[i1-1][i2] = iCurPlayer; // Adjust bomb colors on left
        }
        if (i1 < 7)                       // If not against the right wall
        {
          iBombCount[i1][i2]--;           // Remove one bomb
          iBombCount[i1+1][i2]++;         // Add one bomb to the right
          iPlayer[i1+1][i2] = iCurPlayer; // Adjust bomb colors on right
        }
        if (i2 > 0)                       // If not against the top wall
        {
          iBombCount[i1][i2]--;           // Remove one bomb
          iBombCount[i1][i2-1]++;         // Add one bomb to the top
          iPlayer[i1][i2-1] = iCurPlayer; // Adjust bomb colors on top
        }
        if (i2 < 7)                       // If not against the bottom wall
        {
          iBombCount[i1][i2]--;           // Remove one bomb
          iBombCount[i1][i2+1]++;         // Add one bomb to the bottom
          iPlayer[i1][i2+1] = iCurPlayer; // Adjust bomb colors on bottom
        }
      }
}

// Checks to see if the game is over.
// Returns 0 if the game is not over.
// Returns 3 if red has won.
// Returns 4 if blue has won.
int CheckEnd ()
{
  int i1, i2, iResult;

  iResult = 5;

  for (i1 = 0; i1 < 8; i1++)
    for (i2 = 0; i2 < 8; i2++)
    {
      if (iPlayer[i1][i2] == 1) // Square owned by red
      {
        if (iResult == 5) iResult = 3; // If no blue found, red may have won
        if (iResult == 4) iResult = 0; // Otherwise, game still on
      }
      if (iPlayer[i1][i2] == 2) // Square owned by blue
      {
        if (iResult == 5) iResult = 4; // If no red found, blue may have won
        if (iResult == 3) iResult = 0; // Otherwise, game still on
      }
    }

  if (iResult == 5) iResult = 0; // If nothing was found, game still on

  return (iResult);

//  return 0; // Debug code... Game will not end if this is uncommented
}

// Idle function... Used to handle animation
void Idle ()
{

  if ((iMode == 1) || (iMode == 2)) // Only run if an explosion is occuring
  {
    if (iAngMode)
    {
      iStep--;
      if (iStep <= ANGLE_STEPS * .74) iAngMode = 0;
    }
    else
    {
      iStep++;
      if (iStep >= ANGLE_STEPS * .76) iAngMode = 1;
    }
    if (iMode == 1)                // Explosion is expanding
    {
      iExpStep++;                  // Increase radius
      if (iExpStep == 10)          // If radius is at maximum...
      {
        iMode = 2;                 // Set mode to collapse the explosions
        MoveBombs ();              // Move the bombs
      }
    }
    else                           // Explosion is collapsing
    {
      iExpStep--;                  // Decrease radius

      if (iExpStep == 0)           // If radius is at minimum...
      {
        iMode = CheckEnd ();       // Check for the end of the game
        if (iMode == 0)            // If the game is still on...
        {
          if (DetectExplosions ()) // If there are more explosions...
          {
            iMode = 1;             // Set mode to explode again
            iExpStep = 1;          // Set radius to starting value
          }
          else                     // Otherwise, move on to next turn
          {     
            if (iCurPlayer == 1) iCurPlayer = 2;
            else iCurPlayer = 1;
          }
        }
      }
    }
    DisplayMainWin ();
  }
  else
  {
    if (iStep != ANGLE_STEPS * .75)
    {
      iAngMode = 0;
      iStep = ANGLE_STEPS * .75;
      DisplayMainWin ();
    }
  }
}  

// Sets up view transformation, lights, and calls the display function
void DisplayMainWin ()
{
  float fLightPos[4];
  double fTheta;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
                                     // Clear the window

  glLoadIdentity ();                 // Load identity matrix
  
  fTheta = ((2 * PI) / ANGLE_STEPS) * iStep;
                                     // Setup theta for camera rotation

  gluLookAt (fX, fY, fZ, (1 * cos (fTheta)) + fX, fY,
            (1 * sin (fTheta)) + fZ, 0, 1, 0);
                                     // Set camera to rotate from one point

  // Setup the light position to be the same as the camera
  fLightPos [0] = fX;
  fLightPos [1] = fY;
  fLightPos [2] = fZ;
  fLightPos [3] = 1.0;

  // Set light position
  glLightfv (GL_LIGHT0, GL_POSITION, fLightPos);


  MyDisplay ();                      // Call Display Function

  glFlush ();                        // Execute all OpenGL commands

  glutSwapBuffers ();                // Display the back buffer
}

// This function handles a resize event
void ReshapeMainWin (int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
                                     // Resize viewport to full window
   glMatrixMode (GL_PROJECTION);     // Modify the GL_PROJECTION matrix
   glLoadIdentity ();                // Reset matrix

   glFrustum (-1.0, 1.0, -1.0, 1.0, 1.0, 500.0);
                                     // Setup camera viewing area
   glMatrixMode (GL_MODELVIEW);      // Return to the GL_MODELVIEW matrix
}

// This function handles keyboard hit events
void KeyboardHit(unsigned char key, int x, int y)
{
  int i1, i2;

  // Check for quit condition
  if ((key == 'q') || (key == 'Q') || (key == 27)) exit (0);

  if (iMode == 0)  // If accepting player input...
  {
    if (key == 13) // ENTER key
    {
      // Check for legal move... Square must be empty or owned by the player
      if ((iPlayer[iCurX][iCurY] == 0) ||
          (iPlayer[iCurX][iCurY] == iCurPlayer))
      {
        iBombCount[iCurX][iCurY] += 1;      // Add one bomb
        iPlayer[iCurX][iCurY] = iCurPlayer; // Set square ownership
        if (DetectExplosions ())            // If any explosions...
        {
          iMode = 1;    // Set up explosion mode
          iExpStep = 1; // Set explosions radius to start value
        }
        else                                // Otherwise...
                                            // Switch players for next turn
        {
          if (iCurPlayer == 1) iCurPlayer = 2;
          else iCurPlayer = 1;
        }
        DisplayMainWin ();
      }
    }
  }
  else if ((iMode == 3) || (iMode == 4)) // If someone has won...
  {
    if (key == ' ') // If SPACE is hit, reset game
    {
      for (i1 = 0; i1 < 8; i1++)
        for (i2 = 0; i2 < 8; i2++)
        {
          iBombCount[i1][i2] = 0; // Clear the bomb counts
          iPlayer[i1][i2] = 0;    // Clear the ownership
          iExploding[i1][i2] = 0; // Clear out explosions
        }

      iCurX = 4;                  // Set current selected square X position
      iCurY = 4;                  // Set current selected square Y position
      iCurPlayer = 1;             // Set player to red
      iMode = 0;                  // Set mode to player input mode
      DisplayMainWin ();          // Display the window
    }
  }
}

// This function handles special key hit events (arrow keys)
void SpecialHit(int key, int x, int y)
{
  if (iMode == 0) switch (key)  // If accepting player input...
  {
    case GLUT_KEY_UP:           // Move selection up
      iCurY += 1;
      if (iCurY > 7) iCurY = 7;
      DisplayMainWin ();
      break;

    case GLUT_KEY_DOWN:         // Move selection down
      iCurY -= 1;
      if (iCurY < 0) iCurY = 0;
      DisplayMainWin ();
      break;

    case GLUT_KEY_RIGHT:        // Move selection right
      iCurX += 1;
      if (iCurX > 7) iCurX = 7;
      DisplayMainWin ();
      break;

    case GLUT_KEY_LEFT:         // Move selection left
      iCurX -= 1;
      if (iCurX < 0) iCurX = 0;
      DisplayMainWin ();
      break;
  }
}

int main (int argc, char* argv[])
{
  int i1, i2;

  iAngMode = 0;

  // Initialize game data arrays to starting values
  for (i1 = 0; i1 < 8; i1++)
    for (i2 = 0; i2 < 8; i2++)
    {
      iBombCount[i1][i2] = 0;
      iPlayer[i1][i2] = 0;
      iExploding[i1][i2] = 0;
    }

  iCurX = 4;      // Set current selected square X position
  iCurY = 4;      // Set current selected square Y position
  iCurPlayer = 1; // Set current player to red
  iMode = 0;      // Set mode to player input

  iStep = ANGLE_STEPS * .75;         // Step variable determines                                     
                                     // which angle gets displayed
  // Initial camera position
  fX = X;
  fY = Y;
  fZ = Z;

  glutInit(&argc, argv);             // Initialize glut

  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

  glutInitWindowSize(600, 600);      // Make a nice 600x600 window
  glutInitWindowPosition(0, 0);      // Position window in upper left corner

  glutCreateWindow ("Chain Reaction -- CS 450 Final Project");
                                     // Create the window

  glClearColor(0.0, 0.0, 0.0, 1.0);  // Clear the window using black

  glutDisplayFunc (DisplayMainWin);  // Setup the display callback fcn
  glutReshapeFunc (ReshapeMainWin);  // Setup the reshape callback fcn
  glutKeyboardFunc (KeyboardHit);    // Setup the keyboard fcn
  glutSpecialFunc (SpecialHit);      // Setup the special keys fcn
  glutIdleFunc (Idle);               // Setup the idle callback fcn

 
  glEnable(GL_DEPTH_TEST);           // Enable Zbuffering, depth check
  glEnable(GL_LIGHTING);             // Enable Lighting
  glEnable(GL_LIGHT0);               // Turn on the first light
  glEnable(GL_NORMALIZE);            // Enable normalizing normal vectors
  glShadeModel (GL_SMOOTH);          // Enable smooth shading

  glutMainLoop ();                   // Start the main loop

  return 0;                          // Fool the compiler
                                     // This never executes
}